/***

The MIT License

Copyright (c) 2018 Teppei Tamra (TAM)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

***/

#ifndef SAORIVIEW_H
#define SAORIVIEW_H

#include <QWidget>
#include <QList>
#include <QJsonObject>
#include <QJsonArray>
#include <saoriviewentry.h>

namespace Ui {
class SaoriView;
}

class SaoriView : public QWidget
{
    Q_OBJECT

public:
    explicit SaoriView(const QString view,const QString account,QWidget *parent = 0);
    ~SaoriView();
    static SaoriView* findView(const QString view,const QString account);
    static QList<SaoriView*> findView(const QString account);

protected:
    QString m_viewname;
    QString m_account;
    QString m_config;
    QList<SaoriViewEntry*> m_entries;
    qlonglong m_maxid;
    qlonglong m_sinceid;
    int m_timecount;
    QJsonObject m_header;

    static QList<SaoriView*> m_viewList;

protected:
    void reload();
    const QString headerParser(const QJsonObject json,bool followed,bool following,bool muting,bool blocking);
    const QString instanceInfoParser(const QString instance);
    const QString statusParser(const QJsonObject json);
    const QString accountParser(const QJsonObject json);
    const QString mediaParser(const QJsonObject json,bool sensitive = false);
    const QString contentParser(const QString content);
    const QString notificationParser(const QJsonObject json);
    const QString followerParser(const QJsonObject json);

    static const QString htmlDiv(const QString divclass,const QString text);
    static const QString htmlSpan(const QString spanclass,const QString text);
    static const QString htmlImg(const QString type,const QString url);
    static const QString htmlAnc(const QString link,const QString text);
    static const QString htmlHr();

    void savePinnedGeometry();
    virtual void closeEvent(QCloseEvent *event);
    void getIdsFromHeader(const QByteArray header);

public slots:
    void recived(const QString timeline,const QByteArray data,const QByteArray link);
    void linkClicked(const QUrl &url);
    void setAvatar(const QString avatar);

protected slots:
    void timerUpdate();
    void viewEntriesAdjustment();
    void relationship(qlonglong id,QJsonObject data);

private:
    Ui::SaoriView *ui;

signals:
    void openMediaView(const QUrl);
    void openAccountView(const QStringList view);
    void newEntryRecived(const QString view,const QString account,int count);

private slots:
    void on_actionReload_triggered();
    void on_actionNewest_triggered();
};

#endif // SAORIVIEW_H
